@@ -31,6 +31,7 @@
 #include <e-util/e-config.h>
 #include <e-util/e-account-utils.h>
 #include <mail/e-mail-backend.h>
+#include <mail/e-mail-folder-utils.h>
 #include <mail/em-config.h>
 #include <mail/em-folder-properties.h>
 #include <mail/em-folder-tree.h>
@@ -92,7 +93,7 @@ refresh_folder_tree (EMFolderTreeModel *model, CamelStore *store)
 void
 shared_folder_commit (EPlugin *ep, EConfigTarget *tget)
 {
-	EMConfigTargetFolder *target =  (EMConfigTargetFolder *)tget->config->target;
+	EMConfigTargetFolder *target =  (EMConfigTargetFolder *) tget->config->target;
 	CamelStore *parent_store;
 	EMFolderTreeModel *model = em_folder_tree_model_new (); /*mail_component_peek_tree_model (mail_component_peek ());*/
 
@@ -101,7 +102,7 @@ shared_folder_commit (EPlugin *ep, EConfigTarget *tget)
 	if (common) {
 		share_folder (common);
 		refresh_folder_tree (model, parent_store);
-		g_object_run_dispose ((GObject *)common);
+		g_object_run_dispose ((GObject *) common);
 		common = NULL;
 	}
 }
@@ -110,7 +111,7 @@ void
 shared_folder_abort (EPlugin *ep, EConfigTarget *target)
 {
 	if (common) {
-		g_object_run_dispose ((GObject *)common);
+		g_object_run_dispose ((GObject *) common);
 		common = NULL;
 	}
 }
@@ -244,10 +245,10 @@ users_dialog_response (GtkWidget *dialog, gint response, struct ShareInfo *ssi)
 {
 	struct _EMFolderTreeModelStoreInfo *si;
 	EMFolderSelector *emfs = ssi->emfs;
-	const gchar *uri, *path;
-	EMailSession *session;
-	CamelService *service = NULL;
-	CamelURL *url;
+	EMFolderTree *folder_tree;
+	EMailBackend *mail_backend;
+	CamelStore *store;
+	gchar *folder_name;
 
 	if (response != GTK_RESPONSE_OK) {
 		gtk_widget_destroy ((GtkWidget *) emfs);
@@ -255,42 +256,34 @@ users_dialog_response (GtkWidget *dialog, gint response, struct ShareInfo *ssi)
 		return;
 	}
 
-	uri = em_folder_selector_get_selected_uri (emfs);
-	path = em_folder_selector_get_selected_path (emfs);
-	session = em_folder_tree_get_session (emfs->emft);
+	folder_tree = em_folder_selector_get_folder_tree (emfs);
+	mail_backend = em_folder_tree_get_backend (folder_tree);
 
-	d(printf ("Creating new folder: %s (%s)\n", path, uri));
+	if (!em_folder_tree_get_selected (folder_tree, &store, &folder_name))
+		g_return_if_reached ();
 
-	url = camel_url_new (uri, NULL);
-	if (url != NULL) {
-		service = camel_session_get_service_by_url (
-			CAMEL_SESSION (session), url, CAMEL_PROVIDER_STORE);
-		camel_url_free (url);
-	}
-
-	if (!CAMEL_IS_STORE (service))
-		return;
-
-	si = em_folder_tree_model_lookup_store_info (
-		ssi->model, CAMEL_STORE (service));
+	si = em_folder_tree_model_lookup_store_info (ssi->model, store);
 	if (si == NULL) {
 		g_assert_not_reached ();
 		return;
 	}
 
-	if (CAMEL_IS_VEE_STORE (service)) {
+	if (CAMEL_IS_VEE_STORE (store)) {
 		EFilterRule *rule;
 
-		rule = em_vfolder_rule_new (session);
-		e_filter_rule_set_name (rule, path);
+		rule = em_vfolder_rule_new (mail_backend);
+		e_filter_rule_set_name (rule, folder_name);
 		vfolder_gui_add_rule (EM_VFOLDER_RULE (rule));
-		gtk_widget_destroy ((GtkWidget *)emfs);
+		gtk_widget_destroy ((GtkWidget *) emfs);
 	} else {
 		g_object_ref (emfs);
 		ssi->d = dialog;
-		create_folder (si->store, path, new_folder_created_cb, ssi);
-
+		create_folder (
+			si->store, folder_name, new_folder_created_cb, ssi);
 	}
+
+	g_object_unref (store);
+	g_free (folder_name);
 }
 
 static void
@@ -345,27 +338,27 @@ new_folder_response (EMFolderSelector *emfs, gint response, EMFolderTreeModel *m
 void
 gw_new_shared_folder_cb (GtkAction *action, EShellView *shell_view)
 {
+	EMFolderSelector *selector;
 	EMFolderTree *folder_tree;
 	EShellBackend *shell_backend;
-	EMailSession *session;
 	GtkWidget *dialog;
-	gchar *uri;
 	gpointer parent;
 
 	parent = e_shell_view_get_shell_window (shell_view);
 	shell_backend = e_shell_view_get_shell_backend (shell_view);
-	session = e_mail_backend_get_session (E_MAIL_BACKEND (shell_backend));
 
-	folder_tree = (EMFolderTree *) em_folder_tree_new (session);
-	emu_restore_folder_tree_state (folder_tree);
+	dialog = em_folder_selector_create_new (
+		parent, E_MAIL_BACKEND (shell_backend), 0,
+		_("Create folder"), _("Specify where to create the folder:"));
 
-	dialog = em_folder_selector_create_new (parent, folder_tree, 0, _("Create folder"), _("Specify where to create the folder:"));
-	uri = em_folder_tree_get_selected_uri (folder_tree);
-	if (uri != NULL)
-		em_folder_selector_set_selected ((EMFolderSelector *) dialog, uri);
-	g_free (uri);
+	selector = EM_FOLDER_SELECTOR (dialog);
+	folder_tree = em_folder_selector_get_folder_tree (selector);
+
+	g_signal_connect (
+		dialog, "response",
+		G_CALLBACK (new_folder_response),
+		gtk_tree_view_get_model (GTK_TREE_VIEW (folder_tree)));
 
-	g_signal_connect (dialog, "response", G_CALLBACK (new_folder_response), gtk_tree_view_get_model (GTK_TREE_VIEW (folder_tree)));
 	gtk_window_set_title (GTK_WINDOW (dialog), "New Shared Folder" );
 	gtk_widget_show (dialog);
 }
@@ -373,21 +366,21 @@ gw_new_shared_folder_cb (GtkAction *action, EShellView *shell_view)
 GtkWidget *
 org_gnome_shared_folder_factory (EPlugin *ep, EConfigHookItemFactoryData *hook_data)
 {
-
-	gchar *folderuri = NULL;
-	gchar *account = NULL;
 	gchar *id = NULL;
-	gchar *folder_name = NULL;
+	const gchar *folder_name;
 	EGwConnection *cnc;
 	ShareFolder *sharing_tab;
-	EMConfigTargetFolder *target=  (EMConfigTargetFolder *)hook_data->config->target;
+	EMConfigTargetFolder *target=  (EMConfigTargetFolder *) hook_data->config->target;
 	CamelFolder *folder = target->folder;
+	CamelProvider *provider;
+	CamelStore *store;
 
-	folder_name = g_strdup (camel_folder_get_full_name (folder));
-	folderuri = g_strdup (target->uri);
-	if (folderuri && folder_name)
-		account = g_strrstr(folderuri, "groupwise");
-	else
+	folder_name = camel_folder_get_full_name (folder);
+
+	store = camel_folder_get_parent_store (folder);
+	provider = camel_service_get_provider (CAMEL_SERVICE (store));
+
+	if (g_strcmp0 (provider->protocol, "groupwise") != 0)
 		return NULL;
 
 	 /* This is kind of bad..but we don't have types for all these folders.*/
@@ -404,32 +397,26 @@ org_gnome_shared_folder_factory (EPlugin *ep, EConfigHookItemFactoryData *hook_d
 	     && strcmp (folder_name, "Trash")
 	     && strcmp (folder_name, "Checklist"))) {
 
-		g_free (folderuri);
 		return NULL;
 	}
 
-	if (account) {
-		CamelStore *parent_store;
+	cnc = get_cnc (store);
 
-		parent_store = camel_folder_get_parent_store (folder);
-		cnc = get_cnc (parent_store);
+	if (E_IS_GW_CONNECTION (cnc))
+		/* XXX Are we leaking folder_name?  I can't tell. */
+		id = get_container_id (cnc, g_strdup (folder_name));
+	else
+		g_warning("Could not Connnect\n");
 
-		if (E_IS_GW_CONNECTION (cnc))
-			id = get_container_id (cnc, folder_name);
-		else
-			g_warning("Could not Connnect\n");
-
-		if (cnc && id)
-			sharing_tab = share_folder_new (cnc, id);
-		else
-			return NULL;
-
-		gtk_notebook_append_page((GtkNotebook *) hook_data->parent, (GtkWidget *) sharing_tab->vbox, gtk_label_new_with_mnemonic N_("Sharing"));
-		common = sharing_tab;
-		g_free (folderuri);
-		return GTK_WIDGET (sharing_tab);
-	} else
+	if (cnc && id)
+		sharing_tab = share_folder_new (cnc, id);
+	else
 		return NULL;
+
+	gtk_notebook_append_page((GtkNotebook *) hook_data->parent, (GtkWidget *) sharing_tab->vbox, gtk_label_new_with_mnemonic N_("Sharing"));
+	common = sharing_tab;
+
+	return GTK_WIDGET (sharing_tab);
 }
 
 EGwConnection *
